#ifndef _CROSSPLANEFUNCTIONS_H
#define _CROSSPLANEFUNCTIONS_H

//#pragma once
/* Technicolor Confidential - Trade Secret
*/
/************* COPYRIGHT AND CONFIDENTIALITY INFORMATION *********
Copyright (c) 2015 Technicolor R&D France All Rights Reserved This program
contains proprietary information which is a trade secret/business secret of
Technicolor R&D France and is protected, even if unpublished, under applicable
Copyright laws (including French droit d'auteur). Recipient is to retain this
program in confidence and is not permitted to use or make copies thereof other
than as permitted in a written agreement with Technicolor R&D France unless
otherwise expressly allowed by applicable laws or by Technicolor R&D France
under express agreement.

*************************************************************************/

#include <cmath>
#include <algorithm>
#include <vector>

#include "Global.H"
#if TCH_CROSSPLANE_CRCB
class Frame; 

#define TCH_FLOAT_TO_INT_BALANCED 0
struct LogGParam {
  float Ba,gamma,b0;
};

namespace Tch {
  template<typename T> T clip(T x,T m,T M) { return (x<=m)?(m):(x>=M?M:x); }

  namespace kPQ { // constants for PQ
    static const double m1 = (2610.0        ) / (4096.0 * 4.0);
    static const double m2 = (2523.0 * 128.0) / 4096.0;
    static const double c1 = (3424.0        ) / 4096.0;
    static const double c2 = (2413.0 *  32.0) / 4096.0;
    static const double c3 = (2392.0 *  32.0) / 4096.0;
    static const float peak=10000.;
  }

  inline double PQi(double Y) {
    using namespace kPQ;
    Y=clip(Y,0.,1.);
    double tempValue = pow(Y, (1.0 / m2));
    return clip((pow(std::max(0.0, (tempValue - c1))/(c2 - c3 * tempValue),(1.0 / m1))),0.,1.);
  }


  inline double logG(double L,LogGParam prm, const float peakContent) {

    L=clip(L,0.,1.);
    const double Ba_n=prm.Ba/peakContent;
    const double max_out = log(1. + pow(1./Ba_n, 1. / prm.gamma) / prm.b0);

    const double Y = log(1. + pow(L / Ba_n,(double)(1/prm.gamma)) / prm.b0 )/max_out;
    return clip(Y,0.,1.);
  }


  inline std::vector<float> logG_PQi(LogGParam prm, const float peakContent) {
    const int N=1024;
    std::vector<float> v(N);
    //    const double Ba_n=prm.Ba/peakContent;
    for(int Y=0;Y<N;++Y) {
      const double Y_n=(double)Y/(N-1); // [0..1]
      v[Y]=(float)logG(PQi(Y_n)*kPQ::peak/peakContent,prm, peakContent);
    }
    return v;
  }
}
#endif
#endif //_CROSSPLANEFUNCTIONS_H
