/* The copyright in this software is being made available under the BSD
 * License, included below. This software may be subject to other third party
 * and contributor rights, including patent rights, and no such rights are
 * granted under this license.
 *
 * <OWNER> = Apple Inc.
 * <ORGANIZATION> = Apple Inc.
 * <YEAR> = 2014
 *
 * Copyright (c) 2014, Apple Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *  * Neither the name of the <ORGANIZATION> nor the names of its contributors may
 *    be used to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

/*!
 *************************************************************************************
 * \file Defines.H
 *
 * \brief
 *    Simple defines used throughout the entire system
 *
 * \author
 *     - Alexis Michael Tourapis         <atourapis@apple.com>
 *
 *************************************************************************************
 */

#ifndef __Defines_H__
#define __Defines_H__

#include <stdint.h>

#define VERSION "0.10_HDRETM_0.11-dev"
#define UNDEFINED 0x7fffffff
#define INT_INF 0x7fffffff
#define FLT_INF 1e30F

#define HDR_ETM_RESHAPER  1        // main control for HDR ETM reshaper implementations
#if HDR_ETM_RESHAPER

#define ARRS_MODEL_OPTIMIZATION     1
#if ARRS_MODEL_OPTIMIZATION
  #define CHROMA_INV_LUt_SCALE        10000.0f
  #define ARRS_MATCH_INVERSE          0
  #define RESHAPER_INPUT_BD_INC       0
  #define MAX_LUMA_FORWARD_RESHAPING_LUT_SIZE (1<<(10+RESHAPER_INPUT_BD_INC))
  #define RESHAPER_INPUT_SAMPLE_RNAGE 1     // 0:STD range, 1: Full range, etc.
  #define STAT_BLOCK_SIZE             16
  #define COLOCATED_LUMA_MODE         1     // 0: top-left 1: average of 4 2: weighted average of 4
  #define MIN_CROSS_LAYER_RESHAPE_SCALE       0.2f
  #define MAX_CROSS_LAYER_RESHAPE_SCALE       5.0f
//Perceptual constants
  #define MASKING_FACTOR              0.001f
  #define ADAPTATION_RATE             0.5f
  #define BLACK_ADAPTATION_FACTOR     0.001f
  #define GAMMA_LIFT                  0.5f
  #define GAMMA_CORRECTION_MIN        1.0f
  #define GAMMA_CORRECTION_MAX        2.0f
  #define SCENE_CHANGE_THR            0.1f
#endif

#define TCH_CROSSPLANE_CRCB 1
#if TCH_CROSSPLANE_CRCB
  #define TCH_AVERAGE_Y_COLOC_MODE          1  // 0:Top-Left 1:Average-4
  #define TCH_BA_WRITE_THRESHOLD            0.15
  #define TCH_PWMODEL_ATF_DOMAIN1           10
  #define TCH_DEFAULT_PRECISION             4
  #define TCH_GLOBAL_OFFSET_SHIFT           4
  #define TCH_LUMINANCE_MIN                 0.005
  #define TCH_ATF_SCALE_BIT_DEPTH			10
  #define TCH_ATF_SCALE_FRAC_BIT_DEPTH		5
  #define TCH_ATF_OFFSET_BIT_DEPTH			10
  #define TCH_ATF_OFFSET_FRAC_BIT_DEPTH		10
  #define TCH_NB_PIECES_PWL                 8
  #define TCH_BETA_COMPUTE_STEP             128 // recommended : = 1024/TCH_NB_PIECES_PWL
#endif

#define QC_CBCR_m37064       1    // Histogram scaling proposed by QC
#if QC_CBCR_m37064
  #define QC_ATF_SCALE_BIT_DEPTH			10
  #define QC_ATF_SCALE_FRAC_BIT_DEPTH		8
  #define QC_ATF_OFFSET_BIT_DEPTH			10
  #define QC_ATF_OFFSET_FRAC_BIT_DEPTH	10
#endif

#define DI_RESHAPER_m37269_m37267       1     //Luma reshaping proposed by DLB and IDCC (m37269), automatic luma reshaping model (m37267)
#if DI_RESHAPER_m37269_m37267
  #define MAX_N_SEGMENTS                (8)
  #define MIN_N_SEGMENTS                (3)   // assuming first flat, middle, lasts flat in extreme case note 1-segment linear will use three segments to describe
  #define MAX_LUMA_RESHAPING_LUT_SIZE   (1024)
  #define MAX_LUMA_RESHAPE_SEGMENTS     (MAX_N_SEGMENTS)
  #define NUMBER_PIVOT_STRATEGIES       (2)   // number of candidate pivot point strategies
  #define MIN_PIVOT_SPACING             (4)
  #define MAX_PRED_ORDER		        (2)
  #define MIN_INT_CONSTANT_COEFFICIENT  (0)
  #define MAX_INT_CONSTANT_COEFFICIENT  (1023)
  #define MIN_LINEAR_COEFFICIENT        (-1023)
  #define MAX_LINEAR_COEFFICIENT        (1023)  

  #define LUMA_RESHAPE_METHOD                1    //new automatic luma reshaping method for CE2
  #if LUMA_RESHAPE_METHOD
	#define MAX_LUMA_HIST_BINS              (32)     
	#define NO_INTRA_PERIOD_UPDATE           1    // improvement of metadata update 
	#define RESHAPER_JOINT_OPT               1
  #endif
#endif


// other general bug fixes of original HDRTools software
#define FULL_RANGE_FIX          1
#define EXR_OUTPUT_CLIPPING     1   

#define MODE0_UPDATE            1
#define MODE1_UPDATE            1
#endif


#ifndef __SIM2_SUPPORT_ENABLED__
//  #define __SIM2_SUPPORT_ENABLED__
#endif


#if (defined(WIN32) || defined(WIN64)) && !defined(__GNUC__)
typedef unsigned char       imgpel;                 //!<  Pixel type definition (8 bit)
typedef unsigned char       byte;
typedef unsigned __int8     uint8;
typedef unsigned __int16    uint16;
typedef unsigned __int32    uint32;
typedef unsigned __int64    uint64;
typedef          __int8     int8;
typedef          __int16    int16;
typedef          __int32    int32;
typedef          __int64    int64;
# ifndef INT64_MIN
#  define INT64_MIN        (-9223372036854775807i64 - 1i64)
# endif
#else
typedef unsigned char       imgpel;                 //!<  Pixel type definition (8 bit)
typedef unsigned char       byte;
typedef unsigned char       uint8;
typedef unsigned short      uint16;
typedef unsigned int        uint32;
typedef unsigned long long  uint64;
typedef          char       int8;
typedef          short      int16;
typedef          int        int32;
typedef          long long  int64;
# ifndef INT64_MIN
#  define INT64_MIN        (-9223372036854775807LL - 1LL)
# endif
#endif



// lets define half as float16 and make it ot type unsigned short 
typedef unsigned short float16;

// boolean constants
#ifndef FALSE
enum {
  FALSE = 0,
  TRUE  = 1
};
#endif

// integer constants (helps with searches in code)
enum {
  ZERO  = 0,
  ONE   = 1,
  TWO   = 2,
  THREE = 3,
  FOUR  = 4,
  FIVE  = 5
};

// Color components
enum {
  Y_COMP  = 0,    // Y Component
  R_COMP  = 0,    // R Component
  U_COMP  = 1,    // U Component
  Cb_COMP = 1,
  UP_COMP = 1,    // U' Component
  G_COMP  = 1,    // G Component
  V_COMP  = 2,    // V Component
  Cr_COMP = 2,
  VP_COMP = 2,    // V' Component
  B_COMP  = 2,    // B Component
  A_COMP  = 3,    // Alpha Component
  T_COMP
};



// File format definitions
enum{
  FF_RAW = 0,
  FF_EXR = 1,
  FF_TIF = 2
};

// command line parameter parsing
enum {
  MAX_CL_PARAM_LENGTH = 2000,
  MAX_CL_PARAMS       = 200
};

// various
enum {
  FILE_NAME_SIZE = 255,
  MAX_LINE_LEN = 2000, // maximum length of line in config file
  MAX_FILT_PIC = 6     // maximum number of frame filters
};

#endif
