/* The copyright in this software is being made available under the BSD
 * License, included below. This software may be subject to other third party
 * and contributor rights, including patent rights, and no such rights are
 * granted under this license.
 *
 * Copyright (c) 2010-2015, ITU/ISO/IEC
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *  * Neither the name of the ITU/ISO/IEC nor the names of its contributors may
 *    be used to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

/** \file     TComReshape.h
    \brief    reshaping header and class (header)
*/

#ifndef __TCOMRESHAPE__
#define __TCOMRESHAPE__
#include "CommonDef.h"
//! \ingroup TLibCommon
//! \{
#if HDR_ETM_RESHAPER

#if CHORMA_DQP_IMP
struct fixedPrec{
    int val;
    int fracBitDepth;
    fixedPrec() : val(0), fracBitDepth(0) {}
    fixedPrec(int x, int y) : val(x), fracBitDepth(y) {}

    fixedPrec operator+(const fixedPrec &x) {
        assert( x.fracBitDepth == fracBitDepth);
        return fixedPrec(val + x.val, fracBitDepth);
    }
    fixedPrec operator-(const fixedPrec &x) {
        assert( x.fracBitDepth == fracBitDepth);
        return fixedPrec(val - x.val, fracBitDepth);
    }
    fixedPrec operator*(const fixedPrec &x) {
        return fixedPrec(val * x.val, fracBitDepth + x.fracBitDepth);
        return *this;
    }
    fixedPrec operator/(const fixedPrec &x) {
        return fixedPrec(val / x.val, fracBitDepth - x.fracBitDepth);
        return *this;
    }
    fixedPrec & operator=(const fixedPrec &x) { val = x.val; fracBitDepth = x.fracBitDepth; return *this;}

    bool operator<(const fixedPrec &x) { assert( fracBitDepth == x.fracBitDepth); return val < x.val; }
    bool operator<=(const fixedPrec &x) { assert( fracBitDepth == x.fracBitDepth); return val <= x.val; }
    bool operator>(const fixedPrec &x) { assert( fracBitDepth == x.fracBitDepth); return val > x.val; }
    bool operator>=(const fixedPrec &x) { assert( fracBitDepth == x.fracBitDepth); return val >= x.val; }
    bool operator==(const fixedPrec &x) { return (val == x.val) && (fracBitDepth == x.fracBitDepth) ; }

    static fixedPrec rshift(fixedPrec const var, int x) { return fixedPrec(var.val << x, var.fracBitDepth + x);}
};
#endif

typedef struct _reshape_info
{
  Int POC;
  // bitstream syntax values
  Int luma_bit_depth;  
  Int reshape_equal_ranges;
  Int reshapePresentFlag;
  Int reshape_num_ranges_minus1;
  Int reshape_global_offset_val;
  Int reshape_range_val[MAX_LUMA_RESHAPE_SEGMENTS];
  Int coeff_log2_offset_minus2;  
  Int reshape_continuity_flag;
  Int constantCoeffSignalledInt[MAX_LUMA_RESHAPE_SEGMENTS+1];
  Int constantCoeffSignalledFrac[MAX_LUMA_RESHAPE_SEGMENTS+1];
  Int linearCoeffSignalledInt;
  Int linearCoeffSignalledFrac;
  // derived values
  Int fractionBits[MAX_LUMA_PRED_ORDER+1];
  Int constantCoeffSignalled[MAX_LUMA_RESHAPE_SEGMENTS+1];
  Int linearCoeffSignalled;
  Int nAllSegmentLengths;
  Int allSegmentLengths[MAX_LUMA_RESHAPE_SEGMENTS];
  Bool firstSegmentFlat;
  Bool lastSegmentFlat;
  Int64 polyCoeff[MAX_LUMA_RESHAPE_SEGMENTS][MAX_LUMA_PRED_ORDER+1];

  Bool  reshapeChromaCrosschannelFlag;
  Int   reshapeScaleIntBitDepth;
  Int   reshapeOffsetIntBitDepth;
  Int   reshapeScaleFracBitDepth;
  Int   reshapeOffsetFracBitDepth;
  Bool  reshapeNegativeScalesPresentFlag;
  Int   reshapeNumComps;

  Int   reshapeNumRanges[3];
  Bool  reshapeEqualRangesFlag[3];
  Int   reshapeGlobalOffsetVal[3];
  Int   reshapeRangeVal[3][MAX_CHROMA_RESHAPE_SEGMENTS];
  Int   reshapeScaleVal[3][MAX_CHROMA_RESHAPE_SEGMENTS];
  Int   reshapeOffset1[2];
  Int   reshapeOffset2[2];

  Int   atfId;
  Int reshapeScaleBitDepth;
  Int reshapeOffsetBitDepth;


  // helping values
  Int sceneNum;
  Int inverseReshapingLUT[MAX_LUMA_RESHAPING_LUT_SIZE];
  Int inverseReshapingChromaLUT[2][MAX_LUMA_RESHAPING_LUT_SIZE];
#if DLB_LUMA_DQP
  Int forwardReshapingLUT[MAX_LUMA_RESHAPING_LUT_SIZE];
#endif

  Int reshaperRangeId; 
  UInt reshaperATFId;

#if RESHAPER_JOINT_OPT 
  Int opt;
  Int dQP[15];
#endif

} ReshapeInfo;

// ====================================================================================================================
// Class definition
// ====================================================================================================================

/// reshaping class

class TComReshape
{

public:
  ReshapeInfo m_ReshapeInfo;

  Void DetermineAllFractionBits(Int coeff_log2_offset_minus2,Int bitDepth,Int fractionBits[MAX_LUMA_PRED_ORDER+1]);
  Void DeriveAllSegments(Int nSegmentSignaled,Int SegmentLengths[MAX_LUMA_RESHAPE_SEGMENTS],Int global_offset_val,Int total_length,Int *pNumberAllSegments,Int pAllSegmentLengths[MAX_LUMA_RESHAPE_SEGMENTS],Bool *pFirstSegmentFlat,Bool *pLastSegmentFlat);
  Void DeriveSmoothPolynomialCoefficients(Int nSegmentSignaled, Int SegmentLengths[MAX_LUMA_RESHAPE_SEGMENTS], Int constantCoeff[1+MAX_LUMA_RESHAPE_SEGMENTS], Int linearCoeff, Bool FirstSegmentFlatFlag, Bool LastSegmentFlatFlag,Int nFractionBits[MAX_LUMA_PRED_ORDER+1],Int64 polyCoeff[MAX_LUMA_RESHAPE_SEGMENTS][MAX_LUMA_PRED_ORDER+1]);
  Void DerivePiecewiseLinearPolynomialCoefficients(Int nSegmentSignaled, Int SegmentLengths[MAX_LUMA_RESHAPE_SEGMENTS], Int constantCoeff[1+MAX_LUMA_RESHAPE_SEGMENTS],Bool FirstSegmentFlatFlag,Bool LastSegmentFlatFlag,Int nFractionBits[MAX_LUMA_PRED_ORDER+1],Int64 polyCoeff[MAX_LUMA_RESHAPE_SEGMENTS][MAX_LUMA_PRED_ORDER+1]);

  Void BuildInverseReshapingLUT(Int nSegmentAll, Int allSegmentLengths[MAX_LUMA_RESHAPE_SEGMENTS],Int nFractionBits[MAX_LUMA_PRED_ORDER+1], Int64 polyCoeff[MAX_LUMA_RESHAPE_SEGMENTS][MAX_LUMA_PRED_ORDER+1],Int lut_size,Int InverseReshapingLUT[MAX_LUMA_RESHAPING_LUT_SIZE]);
  Void BuildForwardReshapingLUT(Int InverseReshapingLUT[MAX_LUMA_RESHAPING_LUT_SIZE],Int ForwardReshapingLUT[MAX_LUMA_RESHAPING_LUT_SIZE],Int lut_size);
  Void BuildIdentityLUT(Int LUT[MAX_LUMA_RESHAPING_LUT_SIZE],Int lut_size);

  Void WriteCoefficientData(FILE *fp,ReshapeInfo *pInfo);
  Void WriteMetaData(FILE *fp,ReshapeInfo *pInfo);
  Void ReadMetaData(FILE *fp, ReshapeInfo *pInfo);
  Int evaluateIntRepn(Float const x, Int const numBitsVal, Int const numBitsFrac);
  Float evaluateFloatRepn(Int x, Int const numBitsVal, Int const numBitsFrac);
  Void ReadChromaMetaData(FILE *fp,ReshapeInfo *pInfo);
  Int evaluateIntRepnTCH(Float const x, Int const numBitsVal, Int const numBitsFrac);
  Float evaluateFloatRepnTCH(Int x, Int const numBitsVal, Int const numBitsFrac);

#if CHORMA_DQP_IMP
	Void identifyComponentRangeIdx(Pel sampleVal, Int shiftVal, std::vector<fixedPrec> &rangeValues,  Int &idxS);
	Void buildInverseReshapingCrossPlane(ReshapeInfo const *pInfo, std::vector<fixedPrec> *inputRanges, std::vector<fixedPrec> *scaleValues, std::vector<fixedPrec> *offsetValue, Int *chromaLutU, Int *chromaLutV);
#endif
};// END CLASS DEFINITION TComReshape

//! \}
#endif // HDR_ETM_RESHAPER
#endif // __TCOMRESHAPE__


